/**************************************************************************/ 
/*                                                                        */ 
/*          Copyright (c) 2002 by Control Technology Corporation          */ 
/*                          All Rights Reserved                           */ 
/*                                                                        */ 
/*  This software is copyrighted by and is the sole property of Control   */ 
/*  Technology Corporation.  All rights, title, ownership, or other       */ 
/*  interests in the software remain the property of Control Technology   */ 
/*  Corporation.  Any unauthorized use, duplication, transmission         */ 
/*  distribution, or disclosure of this software is expressly forbidden.  */ 
/*                                                                        */ 
/*  This Copyright notice may not be removed or modified without prior    */ 
/*  written consent of Control Technology Corporation.                    */ 
/*                                                                        */ 
/*  Control Technology Corporation reserves the right to modify this      */ 
/*  software without notice.                                              */ 
/*                                                                        */ 
/*  Control Technology Corporation           info@ctc-control.com         */
/*  25 South Street                          http://www.ctc-control.com   */
/*  Hopkinton, MA  02038                                                  */
/*                                                                        */
/**************************************************************************/
/* MODULE: CoreFunc.h                                                     */
/*                                                                        */ 
/* MODULE DESCRIPTION:                                                    */
/*                                                                        */ 
/* Base include module required for all User Function files.  Defines and */
/* includes any other Quickstep specific include files.  Such as          */
/* 1.  Protype declaration of Quickstep OS function calls which first     */
/*     verify table existance prior to allowing access.                   */
/* 2.  tx_api.h - Thread X include files                                  */
/* 3.  Errors.h - Quickstep OS RETVAL function return value definitions   */
/* 4.  logcodes.h - Debug specific logging codes                          */
/* 5.  uTable.h - Virtual function table definitions                      */  
/* 6.  MALLOC_DATA_SIZE - Memory Heap size for dynamic allocation         */
/*     (malloc/free)                                                      */
/* 7.  coreTable - Base Quickstep OS virtual table global references      */
/* 8.  printf - definition for redirection to UDP debugger                */      
/*                                                                        */ 
/* REVISION HISTORY:                                                      */
/*                                                                        */ 
/**************************************************************************/
#include <stdarg.h>

#ifndef	__CORE_FUNCTIONS_H
#define	__CORE_FUNCTIONS_H
#ifdef   __cplusplus
extern   "C" {
#endif

/* Include the log codes, making sure we are not defining the table */
#ifdef APP_EVENT_TABLE_CREATE
#undef APP_EVENT_TABLE_CREATE
#endif
#define APP_EVENT_TABLE_CREATE 0
#include "tx_api.h"

typedef unsigned char   UINT8;
typedef unsigned short  UINT16;
typedef unsigned long   UINT32;
typedef signed char     INT8;
typedef signed short    INT16;
typedef signed long     INT32;

typedef unsigned long   STDVAL;
typedef void            TASK;
typedef unsigned int    INDX;
//typedef unsigned long   SYSMODE;

typedef enum {START_INIT = 0, /*system is performing start-up initialization*/
              RESET,          /*system is resetting*/
              AT_RESET,       /*system reset complete*/
              RESET_INPUT,    /*system reset input detected*/
              RESETSTOP_INP,  /*system reset input and stop input/command detected*/
              RESTART,        /*system is resetting and will "restart" to running*/
              STOPPED,        /*system stopped*/
              PROGRAMMING,    /*system in programming mode*/
              RUNNING,        /*system is running*/
              FAULT,          /*system running has encountered a fault*/
              FAULT_STOPPED,  /*system stopped after encountering a fault*/
              SHUTDOWN,        /*system commanded to shutdown*/
			  OLDSTATEEMPTY   /* used by safe state logic to know when can save current state */
              } SYSMODE;


#include "Errors.h"		// RETVAL return type definitions
#include "logcodes.h"	// UDP Debugger log type definitions
#include "uTable.h"		// Virtual table definitions

#ifndef NULL
#define NULL 0
#endif

#ifndef	TRUE
#define	TRUE	1
#endif

#ifndef	FALSE
#define	FALSE	0
#endif

#ifndef	CRLF
#define	CRLF	"\n\r"
#endif

// Amount of heap memory to reserve for malloc calls
#define MALLOC_DATA_SIZE		(0x20000)

extern void initCoreFunction(void);		// This function must be called during main() to init Quickstep OS table pointer
extern const VTABLE_QS *coreTable;      // Table point to get initialized at the start of exection

// printf declaration to redirect output to Quickstep OS UDP debugger
// DO NOT CHANGE or use the standard library printf statement!
#define printf	if (coreTable) (*coreTable->printf)

/********* Register Access ***************/
RETVAL regRead(UINT16 RegNum, INT32 *RegVal);
RETVAL regWrite(UINT16 RegNum, INT32 RegVal);

/********* Communications ****************/
	/* This function sends a message (undefined format) of length "size" out the specified "port" using the 
       appropriate driver. 
    */
RETVAL commSendMsg( INDX port, UINT8 *msg, UINT16 size);
	/* This function sends a generic message command (undefined format) of length "size" to the 
       specified "port" using the appropriate driver. Any response is copied back to "msg"; the size of the
       of the response is copies back to "size"
    */
RETVAL commGenericCmd( INDX port, UINT8 *msg, UINT16 *size);
    /* This function invokes the driver associated with "port" and returns a pointer to the buffer that contains the 
       request message.  The message contents are NOT copied.  If a request message from the port is not available, the
       "buf" pointer is set to NULL.  "size" will have the number of bytes in the buffer stored in it, this is not
       necessarily the same as the number of bytes in the message.
    */
RETVAL commReadMsg( INDX port, UINT8 **buf, UINT16 *size);
int commNetworkOpen( USERCONNECTION *user);
void commNetworkClose( USERCONNECTION *user);
int commNetworkSend( USERCONNECTION *user);

/************ Motion Routines ****************/
UINT16 Motion_GetNumMotors(void);
RETVAL Motion_Get_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL *value);
RETVAL Motion_Put_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL value);
RETVAL Motion_Ready( UINT8 MotorNum, STDVAL *value);
RETVAL Motion_Simple_Command( UINT8 MotorNum, UINT8 Command);

/*********** System Functions ***************/
unsigned long systemTics(void);		// number of timer tics since powerup, 1ms/tic currently
void *addResourceFilter(RESOURCE_INFO *rsc, FILTER_FUNCTION func);	// add a resource filter function
RETVAL removeResourceFilter(void *handle);		// remove a resource filter function
RETVAL fireWatchdog(void);		// watchdog timer must be reset every 85ms else call this

/*********** Base Threading Functions (Advanced users only) ************/
unsigned int _tx_thread_create( TX_THREAD *thread_ptr,       /* Task Structure */
						/* Task Name */
                        char *name_ptr,
						/* Routine and Parameter to pass */
                        void (*entry_function)(unsigned long), unsigned long entry_input,
                        /* Stack start and length */
                        void *stack_start, unsigned long stack_size,
                        /* Priority and Threshold */
                        unsigned int priority, unsigned int preempt_threshold, 
						/* time slice */
                        unsigned long time_slice, unsigned int auto_start);

unsigned int _tx_thread_suspend(TX_THREAD *thread_ptr);  // suspend a thread
unsigned int _tx_thread_resume(TX_THREAD *thread_ptr);   // resume a suspended thread
unsigned int _tx_thread_sleep(unsigned long tics);	// amount of time to sleep, only call from a thread
unsigned int _tx_mutex_create(TX_MUTEX *mutex_ptr, char *name,unsigned int inherit);  // create a mutex
unsigned int _tx_mutex_get(TX_MUTEX *mutex_ptr, unsigned long wait_option);  // get control of a mutex
unsigned int _tx_mutex_put(TX_MUTEX *mutex_ptr);	// give up control of a mutex
unsigned int _tx_mutex_delete(TX_MUTEX *mutex_ptr);	// delete a mutex

// diagnostics
RETVAL logEvent(EventCode event, long parameter);
RETVAL logEventPlus(EventCode event, long firstParameter, long secondParameter);
int sprintf(char *buffer, const char *format, ...);
int vsprintf(char *buffer, const char *format, va_list ap);


// memory allocation for malloc support
unsigned int _tx_byte_allocate(TX_BYTE_POOL *pool_ptr,
	void **memory_ptr,unsigned long memory_size,unsigned long wait_option); // give malloc memory back
unsigned int _tx_byte_release(void *memory_ptr); // give malloc memory back
unsigned int _tx_byte_pool_create(TX_BYTE_POOL *pool_ptr,
	char *name_ptr, void *pool_start, unsigned long pool_size);
unsigned int _tx_byte_pool_delete(TX_BYTE_POOL *pool_ptr);
unsigned int _txe_byte_pool_delete(TX_BYTE_POOL *pool_ptr);
// interrupt control
//unsigned int disableInts(unsigned int);			// turn interrupts off
//unsigned int enableInts(unsigned int);			// turn interrupts on

// Step control
RETVAL enterStepAtomicity();
RETVAL exitStepAtomicity();

/* Communications Command Requests */
#define COMMCMD_CMND_RESET   0
#define COMMCMD_GENERIC     12
#define COMMCMD_RQST_CTCBIN   1
#define COMMCMD_RQST_MSG      2
#define COMMCMD_RQST_QUERY    3
#define COMMCMD_RQST_GETCH    4
#define COMMCMD_RQST_GETNUM   5
#define COMMCMD_RQST_GETCNT   6
#define COMMCMD_RQST_CLRBUF   7
#define COMMCMD_RQST_NEWBAUD  8
#define COMMCMD_RQST_PARSING  9
#define COMMCMD_RQST_DEVCNET 10
#define COMMCMD_RQST_VERSION 11
#define COMMCMD_RQST_SET_MODBUS 13

/* Communications Command Responses */
#define COMMCMD_RSPN_OK       0
#define COMMCMD_RSPN_BUSY     1
#define COMMCMD_RSPN_READY    2
#define COMMCMD_RSPN_ACK     64
#define COMMCMD_RSPN_NACK    65

#define COM1 1
#define COM2 2

#ifdef   __cplusplus
        }
#endif

#endif	//	__CORE_FUNCTIONS_H
