/*
    Quickstep Core Function

  This module contains the routines for the User C application
  to access the Quickstep Core Functions via the table

*/

#include "CoreFunc.h"

/*
***********************************************************
Local variables
***********************************************************
*/

const VTABLE_QS *coreTable;

#define COREBASE   63

const VTABLE_QS *getCoreAddress(void)
{
	register const VTABLE_QS *address;

	asm volatile (
		"trapa  %1;         "
		"mov    r0,%0; "
		: "=r" (address)      // output in address
		: "i" (COREBASE)      // input is trap number
	: "r0");              // uses r0

	return(address);
}

void initCoreFunction(void)
{
	coreTable = getCoreAddress();
}

const VTABLE_QS *getCoreTable(void)
{
	if (coreTable == NULL)
	{
		coreTable = getCoreAddress();
	}

	return(coreTable);
}

/************************* Register Access ********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regRead                                             PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Read a Quickstep Register value.                                    */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 0 to 64535 to read             */ 
/*    INT32 *RegVal - Pointer to a 32 bit wide Integer to store the result*/ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL regRead(UINT16 RegNum, INT32 *RegVal)
{
	//	REGISTER_GET            regRead;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regRead != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regRead)(NULL, RegNum, RegVal));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regWrite                                            PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Write a value to a Quickstep Register                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 0 to 64535 to read             */ 
/*    INT32  RegVal - 32 bit wide Integer value to store                  */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL regWrite(UINT16 RegNum, INT32 RegVal)
{
	//	REGISTER_PUT            regWrite;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regWrite != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regWrite)(NULL, RegNum, RegVal));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/************************* Communications *********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commSendMsg                                         PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Send a buffer of characters out a serial communications port        */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 *msg - Pointer to unsigned character buffer containing message*/ 
/*    UINT16 size - Length of message to send                             */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */
/*      ERROR_NO_COMM_PORT = Not a defined COMM port                      */ 
/*      ERROR_NOT_DEFINED  = Quickstep OS table not found                 */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL commSendMsg( INDX port, UINT8 *msg, UINT16 size)
{
	//	COMM_SENDMSG            commSendMsg;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commSendMsg != NULL)
		{
			return((*coreTable->commSendMsg)(port, msg, size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commGenericCmd                                      PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function sends a generic message command (undefined format) of*/
/*     length "size" to the specified "port" using the appropriate driver.*/
/*     Any response is copied back to "msg"; the size of the of the       */
/*     response it copies back to "size".                                 */
/*     Messages can be used to change baud rate, etc...                   */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 *msg - Pointer to unsigned character buffer containing message*/ 
/*    UINT16 *size - Pointer to unsigned short to store result length in  */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
/*  Below are msg[] contents commands for each available.  The command must*/
/*  be the first byte of msg[] upon calling the function.
COMMCMD_RQST_QUERY - READ
	Check if the transmitter is free for message sending.
	Upon return:
	msg[0] = 0x00 if free.
	msg[1] = 0x01 if busy.
	*size = 1.
COMMCMD_RQST_CLRBUF - WRITE
	Clear the communications receive buffer.
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_PARSING - WRITE
	Turn parsing on or off based on the third byte of the message.
	Set msg to:
	msg[2] = 0 then disable parsing
	msg[2] = 1 then enable parsing
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_GETCNT - READ
	Return the current receive buffer count.
	Upon return:
	msg[0] = current count
	*size = 1;
COMMCMD_RQST_GETCH - READ
	Retrieve the nth character in the receive buffer.
	Set msg to:
	msg[2] = offset in buffer with 0 being first character
	Upon return:
	msg[0] = character at position requested
	*size = 1;
COMMCMD_RQST_SET_MODBUS - WRITE
	Activate/Deactivate Serial port modbus and set global port to use.
	Set msg to:
	msg[2] = Modbus RTU serial port address 1 to 254, also enables, 0 disables
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_NEWBAUD - WRITE
	Change the baud rate on the serial port (only physical, not virtual work)
	msg[2] = baud rate desired where 1 = 600, 2 = 1200, 3 = 2400, 4 = 4800,
	         5 = 9600, 6 = 19200 (default at powerup), 7 = 38400.
	Upon return:
	msg[0] = COMMCMD_RSPN_NACK if bad value, or COMMCMD_RSPN_ACK if OK
	*size = 1;
*/
RETVAL commGenericCmd( INDX port, UINT8 *msg, UINT16 *size)
{
	//	COMM_GENERICCMD         commGenericCmd;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commGenericCmd != NULL)
		{
			return((*coreTable->commGenericCmd)(port, msg, size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commReadMsg                                         PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function invokes the driver associated with "port" and        */
/*     returns a pointer to the buffer that contains the request message. */
/*     The message contents are NOT copied.  If a request message from    */
/*     the port is not available, the "buf" pointer is set to NULL.  The  */
/*     message contents are assumed to be encoded in one of the standard  */
/*     protocols.  Size is set to the number of received bytes in the     */
/*     buffer, not the number of bytes in the message.                    */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 **buf - Pointer to unsigned character buffer stored here or   */ 
/*                  NULL if no message                                    */
/*    UINT16 *size - Number of bytes in buffer                            */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NO_COMM_PORT = Not a defined COMM port                      */ 
/*      ERROR_NOT_DEFINED  = Quickstep OS table not found                 */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL commReadMsg( INDX port, UINT8 **buf,UINT16 *size)
{
	//	COMM_GET_RQST           commReadMsg;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commReadMsg != NULL)
		{
			return((*coreTable->commReadMsg)(port, buf,size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkOpen                                     PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that a Network UDP or TCP port be openned.  */
/*     A RX thread will be spawned to automatically monitor the socket    */
/*     state.  This functions a USERCONNECTION control block that must    */
/*     not be modfied after a successful call and should be treated as    */
/*     a connection HANDLE on other Network calls.  A callback routine    */
/*     must be suppled as one of the USERCONNECTION parameters which will */
/*     be invoked whenever a change of state occurs, including packet     */
/*     reception.                                                         */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure used to pass  connection/socket    */
/*                information, must be initialized prior to call.  Only   */
/*                network type NETWORK_TYPE_UDP currently supported       */
/*  OUTPUT                                                                */ 
/*    int - 0 if success and -1 if failed                                 */ 
/*      user->state set to USER_FAILED or USER_CONNECTING                 */ 
/*      NOTE:  Once user->state becomes USER_CONNECTED this structure     */
/*             may be modifed and used for commNetworkSend operations     */
/*             It is cloned by the network thread.  The copy is what is   */
/*             modified and supplied to the callback function.            */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
int commNetworkOpen( USERCONNECTION *user)
{
	//	COMM_NETWORKOPEN    commNetworkOpen;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkOpen != NULL)
		{
			return((*coreTable->commNetworkOpen)(user));
		}
	}
	return(-1);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkClose                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that an existing network connection/socket  */
/*     created with a call to commNetworkOpen, be closed.  The same       */
/*     USERCONNECTION structure passed to the commNetworkOpen call should */
/*     be passed to this function.                                        */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure used to pass  connection/socket    */
/*                information, must be initialized prior to call.  Only   */
/*                network type NETWORK_TYPE_UDP currently supported       */
/*                Should be same USERCONNECTION passed to commNetworkOpen */
/*                                                                        */
/*  OUTPUT                                                                */ 
/*    NONE                                                                */ 
/*    user->state modified to either USER_FAILED or USER_DISCONNECTED     */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void commNetworkClose( USERCONNECTION *user)
{
	//	COMM_NETWORKCLOSE    commNetworkClose;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkClose != NULL)
		{
			(*coreTable->commNetworkClose)(user);
			return;
		}
	}
	return;
}
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkSend                                     PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that a packet be sent on a connection       */
/*     openned by commNetworkOpen.  The USERCONNECTION control block      */
/*     must contain the proper information.                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure returned by commNetworkOpen with   */
/*          following parameters set as desired:                          */
/*            packetBuf - pointer to buffer to send                       */
/*            length - number of bytes to send                            */
/*            destIp - If UDP, destination IP address                     */
/*            destPort - if UDP, destination port address                 */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    int - number of bytes queued                                        */ 
/*    user->state set to USER_FAILED or USER_DATAQUEUED                   */
/*      NOTE:  Limited queuing is available at the network stack level    */ 
/*             exceeding this will result in packet loss.  Do not send    */
/*             a large number of packets to the host without using a      */
/*             protocol that can verify packets have been transfered.     */
/*             This is typically only a problem if you attempt to sit in a*/
/*             loop continually transmitting data.                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
int commNetworkSend( USERCONNECTION *user)
{
	//	COMM_NETWORKSEND    commNetworkSend;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkSend != NULL)
		{
			return((*coreTable->commNetworkSend)(user));
		}
	}
	return(-1);
}


/************************* Motion Routines ********************************/
/*                                                                        */
UINT16 Motion_GetNumMotors(void)
{
	//	MOTION_GET_NUMMOTORS    Motion_GetNumMotors;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_GetNumMotors != NULL)
		{
			return((*coreTable->Motion_GetNumMotors)());
		}
	}
	return(0);
}

RETVAL Motion_Get_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL *value)
{
	//	MOTION_GET_ATTRIBUTE    Motion_Get_Attribute;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Get_Attribute != NULL)
		{
			return((*coreTable->Motion_Get_Attribute)(MotorNum, Attribute, value));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Put_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL value)
{
	//	MOTION_PUT_ATTRIBUTE    Motion_Put_Attribute;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Put_Attribute != NULL)
		{
			return((*coreTable->Motion_Put_Attribute)(MotorNum, Attribute, value));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Ready( UINT8 MotorNum, STDVAL *value)
{
	//	MOTION_READY            Motion_Ready;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Ready != NULL)
		{
			return((*coreTable->Motion_Ready)(MotorNum, value));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Simple_Command( UINT8 MotorNum, UINT8 Command)
{
	//	MOTION_SIMPLE_COMMAND   Motion_Simple_Command;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Simple_Command != NULL)
		{
			return((*coreTable->Motion_Simple_Command)(MotorNum, Command));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/************************* System Routines ********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    systemTics                                          PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to get the number of milliseconds that have */
/*    occured since powerup.  Note that this will wrap back to 0.         */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    none                                                                */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    unsigned long - number of system tics, in milliseconds since powerup*/ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned long systemTics(void)		// number of timer tics since powerup, 1ms/tic currently
	{
	//	GET_SYSTEM_TICS         systemTics;
	if (getCoreTable() != NULL)
	{
		if (coreTable->systemTics != NULL)
		{
			return((*coreTable->systemTics)());
		}
	}
	return(0);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    fireWatchdog                                        PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to reset the watchdog system timer that     */
/*    will cause a system reset and fault if not invoked every 86 ms.     */ 
/*    Quickstep OS will automatically call this function as required      */ 
/*    but if a User Function runs as a step it must make the call if it   */ 
/*    maintains control too long, preventing Quickstep OS from calling    */
/*    the function.                                                       */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    none                                                                */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL fireWatchdog(void)		
	{
	//	FIREWATCHDOG         fireWatchdog;
	if (getCoreTable() != NULL)
	{
		if (coreTable->fireWatchdog != NULL)
		{
			(*coreTable->fireWatchdog)();
			return (SUCCESS);
		}
	}
	return(ERROR_NOT_DEFINED);
}

/***************************** Step Control *******************************/
/*                                                                        */
RETVAL enterStepAtomicity(void)
{
	//	ENTER_STEP_ATOMICITY    enterStepAtomicity;
	if (getCoreTable() != NULL)
	{
		if (coreTable->enterStepAtomicity != NULL)
		{
			return((*coreTable->enterStepAtomicity)());
		}
	}
	return(ERROR_NOT_DEFINED);
}
RETVAL exitStepAtomicity(void)
{
	//	EXIT_STEP_ATOMICITY     exitStepAtomicity;
	if (getCoreTable() != NULL)
	{
		if (coreTable->exitStepAtomicity != NULL)
		{
			return((*coreTable->exitStepAtomicity)());
		}
	}
	return(ERROR_NOT_DEFINED);
}

/************************* Resource Filter Routines ***********************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    addResourceFilter                                   PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to add a resource filter to the Quickstep   */
/*    list.  The resource to monitor is defined in the RESOURCE_INFO      */
/*    structure and passed as a parameter.  A pointer to the "C" function */
/*    to call upon access is also passed                                  */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    RESOURCE_INFO *rsc - Pointer to structure defining resource and     */
/*       access method upon which to invoke the passed function           */ 
/*    FILTER_FUNCTION func - Pointer to "C" function to call when the     */
/*    parameters of the RESOURCE_INFO structure are satisfied             */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    void *handle - Handle returned by addResourceFilter or NULL if 	  */
/*                   failed                                               */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
/****************************************************************
*  RESOURCE_INFO rsc - addResourceFilter parameter block        *
*																*
*  This parameter block must be filled out prior to registering *
*  a callback function for a Quickstep resource.  A pointer to  *
*  it is passed to the addResourceFilter() function             *
****************************************************************/
/*
typedef struct resourceInfo {
	int type;		// Type of resource to add filter to, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int mode;		// Type of access to invoke filter on when accessed, RESOURCE_READ, RESOURCE_WRITE
	int start;		// Start number of resource (setting a range)
	int end;		// End number of resource, make same as start if only one (setting a range).
} RESOURCE_INFO;
*/
/****************************************************************
* FILTER_FUNCTION func                      			        *
*																*
* Type definition for Resource Callback function			    *
*																*
* PARAMETERS:													*
* void *handle - handle returned by addUserResourceFilter when  *
*                function was registered                        *
* FILTERPARAMS *params - Access information block to detail     *
*                        what is being done                     *
* STDVAL value - Current value being read or written.           *
* RETVAL *status - pointer to status code that will be returned *
*                  to Quickstep.  Leaving it unchanged will     *
*                  default to SUCCESS.  Use only for defined    *
*                  errors.                                      *
*																*
* RETURNS:														*
* int - new value to return to Quickstep on a read or value to  *
*       write, if no change then return passed "value"          *
*****************************************************************/
void *addResourceFilter(RESOURCE_INFO *rsc, FILTER_FUNCTION func)	// add a resource filter function
	{
	//	ADD_RESOURCE_FILTER     addResourceFilter;
	if (getCoreTable() != NULL)
	{
		if (coreTable->addResourceFilter != NULL)
		{
			return((*coreTable->addResourceFilter)(rsc,func));
		}
	}
	return(NULL);
}
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    removeResourceFilter                                PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to remove a resource filter from the        */
/*    Quickstep list.  A filter is removed by passing the handle that     */ 
/*    was returned when it was first added.                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    void *handle - Handle returned by addResourceFilter                 */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL removeResourceFilter(void *handle)		// remove a resource filter function
	{
	//	REMOVE_RESOURCE_FILTER  removeResourceFilter;
	if (getCoreTable() != NULL)
	{
		if (coreTable->removeResourceFilter != NULL)
		{
			return((*coreTable->removeResourceFilter)(handle));
		}
	}
	return(ERROR_NOT_DEFINED);
}
/*************************** Advanced Threading ***************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_create                                   PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function creates a thread and places it on the list of created */ 
/*    threads.                                                            */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Thread control block pointer  */ 
/*    name                                  Pointer to thread name string */ 
/*    entry_function                        Entry function of the thread  */ 
/*    entry_input                           32-bit input value to thread  */ 
/*    stack_start                           Pointer to start of stack     */ 
/*    stack_size                            Stack size in bytes           */ 
/*    priority                              Priority of thread (0-31)     */ 
/*    preempt_threshold                     Preemption threshold          */
/*    time_slice                            Thread time-slice value       */ 
/*    auto_start                            Automatic start selection     */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    return status                         Thread create return status   */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*    _tx_timer_initialize                  Create system timer thread    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_create( TX_THREAD *thread_ptr,       /* Task Structure */
	/* Task Name */
	char *name_ptr,
	/* Routine and Parameter to pass */
	void (*entry_function)(unsigned long), unsigned long entry_input,
	                        /* Stack start and length */
	void *stack_start, unsigned long stack_size,
	                        /* Priority and Threshold */
	unsigned int priority, unsigned int preempt_threshold, 
	/* time slice */
	unsigned long time_slice, unsigned int auto_start)
{
	//	THREAD_CREATE           thread_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_create != NULL)
		{
			return((*coreTable->thread_create)(thread_ptr,name_ptr,entry_function,
				entry_input,stack_start,stack_size,priority,preempt_threshold,time_slice,auto_start));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_suspend                                  PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function handles application suspend requests.  If the suspend */ 
/*    requires actual processing, this function calls the actual suspend  */ 
/*    thread routine.                                                     */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Pointer to thread to suspend  */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Return completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_suspend(TX_THREAD *thread_ptr)  // suspend a thread
{
	//	THREAD_SUSPEND           thread_suspend;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_suspend != NULL)
		{
			return((*coreTable->thread_suspend)(thread_ptr));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_resume                                   PORTABLE C      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function processes application resume thread services.  Actual */ 
/*    thread resumption is performed in the core service.                 */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Pointer to thread to resume   */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Service return status         */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_resume(TX_THREAD *thread_ptr)   // resume a suspended thread
{
	//	THREAD_RESUME           thread_resume;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_resume != NULL)
		{
			return((*coreTable->thread_resume)(thread_ptr));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_sleep                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function handles application thread sleep requests.  If the    */ 
/*    sleep request was called from a non-thread, an error is returned.   */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    timer_ticks                           Number of timer ticks to sleep*/ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Return completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_activate                    Activate sleep timer          */ 
/*    _tx_thread_suspend                    Actual thread suspension      */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*                                                                        */ 
/*    Application code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_sleep(unsigned long tics)	// amount of time to sleep, only call from a thread
	{
	//	THREAD_SLEEP           thread_sleep;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_sleep != NULL)
		{
			return((*coreTable->thread_sleep)(tics));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_create                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function creates a mutex with optional priority inheritance as */ 
/*    specified in this call.                                             */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                             Pointer to mutex control block*/ 
/*    name_ptr                              Pointer to mutex name         */ 
/*    inherit                               Priority inheritance option   */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Successful completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*                                                                        */ 
/*    None                                                                */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_create(TX_MUTEX *mutex_ptr, char *name, unsigned int inherit)
{
	//	MUTEX_CREATE            mutex_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_create != NULL)
		{
			return((*coreTable->mutex_create)(mutex_ptr, name, inherit));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_get                                       PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function gets the specified mutex.  If the calling thread      */ 
/*    already owns the mutex, an ownership count is simply increased.     */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*    wait_option                       Suspension option                 */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    status                            Completion status                 */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_activate                Activate timer routine            */ 
/*    _tx_thread_suspend                Suspend thread service            */ 
/*    _tx_mutex_priority_change         Inherit thread priority           */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_get(TX_MUTEX *mutex_ptr, unsigned long wait_option)
{
	//	MUTEX_GET               mutex_get;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_get != NULL)
		{
			return((*coreTable->mutex_get)(mutex_ptr, wait_option));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_put                                       PORTABLE C      */ 
/*                                                           1.0          */  
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function puts back an instance of the specified mutex.         */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Success completion status         */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_deactivate              Deactivate timer routine          */ 
/*    _tx_thread_resume                 Resume thread service             */ 
/*    _tx_thread_system_return          Return to system routine          */ 
/*    _tx_mutex_priority_change         Restore previous thread priority  */ 
/*    _tx_mutex_prioritize              Prioritize the mutex suspension   */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_put(TX_MUTEX *mutex_ptr)
{
	//	MUTEX_PUT               mutex_put;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_put != NULL)
		{
			return((*coreTable->mutex_put)(mutex_ptr));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_delete                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function deletes the specified mutex.  All threads             */ 
/*    suspended on the mutex are resumed with the TX_DELETED status       */ 
/*    code.                                                               */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Successful completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_deactivate              Deactivate timer routine          */ 
/*    _tx_thread_resume                 Resume thread service             */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_delete(TX_MUTEX *mutex_ptr)
{
	//	MUTEX_PUT               mutex_put;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_delete != NULL)
		{
			return((*coreTable->mutex_delete)(mutex_ptr));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************** Diagnostics *********************************/
/*                                                                        */
RETVAL logEvent(EventCode event, long parameter)
{
	//	LOGEVENT                logEvent;
	if (getCoreTable() != NULL)
	{
		if (coreTable->logEvent != NULL)
		{
			return((*coreTable->logEvent)(event, parameter));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL logEventPlus(EventCode event, long firstParameter, long secondParameter)
{
	//	LOGEVENTPLUS            logEventPlus;
	if (getCoreTable() != NULL)
	{
		if (coreTable->logEventPlus != NULL)
		{
			return((*coreTable->logEventPlus)(event, firstParameter, secondParameter));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/******************** Reserved/not for user access ************************/
/*                                                                        */
unsigned int _tx_byte_allocate(TX_BYTE_POOL *pool_ptr, void **memory_ptr,
	unsigned long memory_size, unsigned long wait_option)
{
	//	BYTE_ALLOCATE           tx_byte_allocate;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_allocate != NULL)
		{
			return((*coreTable->tx_byte_allocate)(pool_ptr, memory_ptr, memory_size, wait_option));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _tx_byte_release(void *memory_ptr)
{
	//	BYTE_RELEASE            tx_byte_release;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_release != NULL)
		{
			return((*coreTable->tx_byte_release)(memory_ptr));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _tx_byte_pool_create(TX_BYTE_POOL *pool_ptr, char *name_ptr, 
	void *pool_start, unsigned long pool_size)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_create != NULL)
		{
			return((*coreTable->tx_byte_pool_create)(pool_ptr, name_ptr, pool_start, pool_size));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _txe_byte_pool_delete(TX_BYTE_POOL *pool_ptr)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_delete != NULL)
		{
			return((*coreTable->tx_byte_pool_delete)(pool_ptr));
		}
	}
	return(TX_POOL_ERROR);
}
unsigned int _tx_byte_pool_delete(TX_BYTE_POOL *pool_ptr)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_delete != NULL)
		{
			return((*coreTable->tx_byte_pool_delete)(pool_ptr));
		}
	}
	return(TX_POOL_ERROR);
}

// interrupt control
unsigned int _tx_thread_interrupt_control(unsigned int state)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->interruptControl != NULL)
		{
			return((*coreTable->interruptControl)(state));
		}
	}
	return(0);
}

/*
***********************************************************
Function access table
***********************************************************
*/
void userShutdown()
{
extern void _systemCleanUp();
void userCleanUp();

logEvent(EVENT_DEBUG,8000);
	// Place any user cleanup here...
	userCleanUp();
	// now shutdown this environment so can allow reload...
	_systemCleanUp();
	// now return to Quickstep, there will be a 2 second delay before starting the load...
}

// Note that any extern's not defined below will automatically be set to 0 by the linker
// as non-entry...
extern int  main(void);
extern int  taskLoop(SYSMODE state);
/*
extern void timerTic(void);
extern int  taskLoop(SYSMODE state);

extern int  superTaskLoop(SYSMODE state);
extern int  serviceLoopHook(SYSMODE state);
extern void endAllTasks(void);
*/
// Below is main table referenced by the Quickstep Operating System, tread carefully when changing
// as it must match that of the OS...  NOTE:  Change version and entries number in table!!!
const VTABLE_USER localUserCommand __attribute__ ((section (".ftable"))) = {
    VALID_USER_SIGNATURE,       // signature
    0x00010008,                 // version 00.01, 8 entries
    main,                       // initialize
    NULL,                   // function unused
    NULL,                   // timerTic;
    taskLoop,                   // taskLoop;
    NULL,                       // superTaskLoop;
    NULL,                       // serviceLoopHook;
    NULL,                     // endAllTasks;
	userShutdown				// userShutdown;
    };

