/**************************************************************************/
/* MODULE: userApp.c                                                      */
/*                                                                        */
/* MODULE DESCRIPTION:                                                    */
/*                                                                        */
/* This module is provided as an example for accessing the Quickstep OS   */
/* from a C programming environment.  The supplied main() example is for  */
/* reference only and must exist somewhere in the users code and it must  */
/* return control in a timely fashion.                                    */
/*                                                                        */
/* REVISION HISTORY:                                                      */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

// Always include CoreFunc.h in any module that will access the Quickstep OS
#include "CoreFunc.h"

#define ESC "\x1B"


// Function prototype
int sampleFilter(void *handle, FILTERPARAMS *params, STDVAL value, RETVAL *status);

/**************************************************************************/
/*                                                                        */
/*  FUNCTION                                               RELEASE        */
/*                                                                        */
/*    main                                                PORTABLE C      */
/*                                                           1.0          */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function is the main input function called after a User        */
/*    C file is loaded into memory for execution.  Any initialization     */
/*    required should be done and control returned.                       */
/*                                                                        */
/*    MAKE SURE TO RETURN CONTROL AND DON"T TAKE LONG!!!  IF CALLED BY    */
/*    A QUICKSTEP RUNNING A SCRIPT, YOU MAY HAVE TO INVOKE A WATCHDOG     */
/*    RESET FUNCTION IF SPEND MORE THAN 40 MS HERE  (fireWatchdog())      */
/*                                                                        */
/*  INPUT                                                                 */
/*                                                                        */
/*    none                                                                */
/*                                                                        */
/*  OUTPUT                                                                */
/*                                                                        */
/*    0 = Initialization successful, allow User functions to execute      */
/*    non-zero = Init failed, do not run User functions                   */
/*                                                                        */
/*  CALLS                                                                 */
/*    - __main() call must be madie prior to any other code               */
/*    - user define initialization routines, as required                  */
/*                                                                        */
/*  CALLED BY                                                             */
/*                                                                        */
/*    C User Function file loader (Quickstep OS)                          */
/*                                                                        */
/*  RELEASE HISTORY                                                       */
/*                                                                        */
/*    DATE              NAME                      DESCRIPTION             */
/*                                                                        */
/*                                                                        */
/**************************************************************************/
int __main();

int main (void)
{
	RESOURCE_INFO resource;
	void *handle;

	__main();

	// Log Event Code to signal we made it here successfully.  This is not
	// required but placed here as test code.  Note all user Events should
	// start at 50000, below that is reserved for Quickstep OS
	logEvent(EVENT_DEBUG, 50000);

	// Send a message to the UDP Debugger screen as though it was our STDIO output
	printf("Hello World.\r\n");

	// Lets install a filter function as a sample
	resource.type = RESOURCE_REGISTER;
	resource.start = 2;	// lets filter register 2
	resource.end = 6;	// no range, only 2 for now
	resource.mode = RESOURCE_READ;	// read operation only, if write too would | RESOURCE_WRITE
	// Now add the filter...
	handle = addResourceFilter(&resource,sampleFilter);

	// if error occurs return non-zero and User functions will not run...
	return(0);
}

// User Filter Example, register 2 to 6 will be divided by 2 when written
int sampleFilter(void *handle, FILTERPARAMS *params, STDVAL value, RETVAL *status)
{
	// This sample filter simply processes a read or write operation on a register
	switch(params->mode)
	{
	case RESOURCE_READ:
		// Someone is attempting to read the register, actual value in in "value"
		// Let's divide it by 2 just for test purposes
		if (value)	// don't divide by 0...
			value = value/2;
		break;
	case RESOURCE_WRITE:
		// Let's not do anything on a write operation, we could have added filter
		// so only reads called this funtion also but this is for future reference
		// on how to process a write
		break;
	}
	// Return new or same value to use
	return value;
}

// User taskLoop Function Example invoked on main Quickstep loop after all steps execute
// NOTE: THIS ONLY RUNS WHEN CONTROLLER IS NOT FAULTED!!!
//       EVEN IF QUICKSET IS NOT USED NEED A SINGLE INSTRUCTION QUICKSTEP PROGRAM IN THE CONTROLLER!
//  userServiceLoopHook is called when faulted and also after every task step
//  you may use the SYSMODE state to determine when taskLoop is not running, possibly only running
//  certain tasks when in FAULT mode, like communications.  Note that if faulted power is 
//  is turned off on the outputs (VBIAS).
int taskLoop(SYSMODE state)
{
	void testSerial(int);
	void testNetwork();
	// Note spending more than 40 ms in this loop will cause a watchdog fault!  Not
	// including time may be pre-empted by interrupts and communication threads
	// fireWatchdog();  will need to be called to reset timer if do.
	// (Watchdog reset upon entry and exit of this function, automatically)
	testSerial(COM1);
	testNetwork();
	return 0;	// always return 0, not used but may be some day
}

// Serial port test function
// commPort = COM1 or COM2 for hardware serial ports
// Register 1 = transmit status/results
// Register 2 = loop counter
void testSerial(int commPort)
{
static int initialized = 0;
static int i = 1;
static int txcnt = 1;
static int errcnt = 0;
RETVAL r;
UINT8 szMsg[3];
UINT16 wSize;
char buf[70];

	if (!initialized)
	{
		initialized = 1;
		// first must disable parsing so get raw data on receiver
		szMsg[0] = COMMCMD_RQST_PARSING;
		szMsg[1] = 0;  /*dummy for port value*/
		szMsg[2] = 0;
		wSize = 3;
		commGenericCmd(commPort, szMsg, &wSize);
	}

	// Invoked here on each loop of quickstep execution
	// Check to see if transmitter is ready
	szMsg[0] = COMMCMD_RQST_QUERY;
	szMsg[1] = 0;  /*dummy for port value*/
	wSize = 2;
	commGenericCmd(commPort, szMsg, &wSize);

	if (szMsg[0] == COMMCMD_RSPN_OK)
	{
		// Serial port ready
		sprintf(buf,"Control Technology Serial Port Test #%d, errors - %d.\r\n",txcnt,errcnt);
		r = commSendMsg(commPort, buf,strlen(buf));
		if (r != SUCCESS)
		{
			// error occured, r = 35 ERROR_NO_COMM_PORT
			errcnt++;
			regWrite(1, r);
		}
		else
		{
			txcnt++;
			regWrite(1, 0);
		}
	}
	else if (szMsg[0] == COMMCMD_RSPN_BUSY)
	{
		// port is busy
		regWrite(1, 999);
	}
	else
	{
		// unknown response
		regWrite(1,888);
	}
	// set loop counter, using register 18
	regWrite(2, i++);
}

/******************* BELOW FOR NETWORK OPERATION **********************/
USERCONNECTION user;	// Structure to define network connections
						// must not be stack variable and must be passed
						// to Open, Send, and Close routines

// Maximum number of packets that can be queued
#define MAX_USER_PACKETS_QUEUED 5
#define MAX_USER_MESSAGE_SIZE 400 // set this to largest message size
// Packet structure for queue
typedef struct
{
	unsigned char packet[MAX_USER_MESSAGE_SIZE+1];	
	int length;
	unsigned long hostIp;
	int hostPort;
} NETWORKPACKET;

// When counts -1 there are no packets to process, when same full
int incount;	// storage count
int outcount;	// retrieval count
NETWORKPACKET packets[MAX_USER_PACKETS_QUEUED];	

TX_MUTEX packetMutex;	// Need to mutex joint resources with main Quickstep thread

// Network Function test program to create a thread to monitor UDP port 7000
// also sends a sample message upon connection
void testNetwork()
{
static int state = 0;
void networkStateChange(void *ptr);

	if (state == 0)
	{
		// first time called therefore open a UDP port, 7000
		// clear everything out
		memset((void *)&user,0,sizeof(USERCONNECTION));
		user.srcPort = 7000;
		user.type = NETWORK_TYPE_UDP;
		incount = outcount = -1;	// set to empty
		// define the function to be invoked by network thread when change of state
		// Note that mutexes must be used within this function if common resources accessed 
		user.StateCallback = networkStateChange;
		printf("attempt open.\r\n");
		// Create a mutex since this thread and network callback are different threads
		 _txe_mutex_create(&packetMutex,"USER C UDP",TX_INHERIT);
		if (commNetworkOpen((USERCONNECTION *)&user))
		{
			// error occurred, cleanup
			printf("Network Open failed\r\n");
		 	_txe_mutex_delete(&packetMutex);
			return;
		}
		printf("open occurred.\r\n");
		// Network thread spawned so now await callback with change of state
		// saying connected.  Can also watch our Control Block.
		state++;
	}
	else if (state == 1)
	{
		// await Network thread to be operational
		if (user.state == USER_CONNECTED)
		{
			printf("got connection.\r\n");
			// we are now ready to run
			state++;
		}
		else if (user.state == USER_DISCONNECTED)
		{
			// connection failed, abort, already closed
			state--;
		}
		else if (user.state == USER_CONNECTING)
		{
			// thread still starting up
		}
		else
		{
			// undefined?????????
		}
	}
	else if (state == 2)
	{
		// lets idle since running, wait for Network data to process
		if (outcount == -1)
		{
			// nothing to do
			return;
		}
		// make sure it is null terminated
		packets[outcount].packet[packets[outcount].length] = 0x00;
		// get next buffer to use, do something
		printf("%s\r\n",packets[outcount].packet); // this sent to UDPterm last heard from on port 1202

		// Test sending a response to the packet
		user.packetBuf = "UDP Packet reception Successful";
		user.length = strlen(user.packetBuf);
		// set response destination to originator of packet
		user.destIp = packets[outcount].hostIp;
		user.destPort = packets[outcount].hostPort;
		commNetworkSend((USERCONNECTION *)&user);

		// must set mutex when bump counter but not needed to process data
		_txe_mutex_get((TX_MUTEX *)&packetMutex, TX_WAIT_FOREVER);
		outcount++;
		if (outcount == MAX_USER_PACKETS_QUEUED)
		{
			outcount = 0;
		}
		if (outcount == incount)
		{
			// we are empty now
			outcount = incount = -1;
		}	
		// unlock resource
		_txe_mutex_put((TX_MUTEX *)&packetMutex);

	}
}

// Callback function to process RX thread state change
void networkStateChange(void *ptr)
{

// It is left to the user to modify the below code for their application

	USERCONNECTION *user = (USERCONNECTION *)ptr;

	switch(user->state)
	{
		case USER_CONNECTED:
			// OK to send now
			printf("Got USER_CONNECTED\r\n");
			break;

		case USER_DISCONNECTED:
			// this connection is has been broken
			printf("Got USER_DISCONNECTED\r\n");
			break;

		case USER_DATA_AVAILABLE:
			// offload data packet, do not access serial ports except on may Quickstep loop
			// the routines is not thread safe
			printf("Got USER_DATA_AVAILABLE, length %d\r\n",user->length);
			// move data to buffers
			_txe_mutex_get((TX_MUTEX *)&packetMutex, TX_WAIT_FOREVER);
			if (incount == -1)
			{
				// this is the first packet
				outcount = 0;
				incount = 0;
			}
			else if (incount == outcount)
			{
				_txe_mutex_put((TX_MUTEX *)&packetMutex);
				return;		// ignore packet, buffers full
			}
			// Move the packet data into our local queues for processing
			memcpy(packets[incount].packet,user->packetBuf,user->length);
			packets[incount].length = user->length;
			packets[incount].hostIp = user->destIp;
			packets[incount].hostPort = user->destPort;
			incount++;
			if (incount == MAX_USER_PACKETS_QUEUED)
			{
				// wrap it
				incount = 0;
			}
			_txe_mutex_put((TX_MUTEX *)&packetMutex);
			break;

		default:
			printf("Got undefined state\r\n");
			return;
	}
}

// Function called when program is being unloaded, possibly new being loaded
void userCleanUp()
{
// must return any allocated memory to system or close network connections if open so threads stop
	if (user.state == USER_CONNECTED)
	{
		commNetworkClose((USERCONNECTION *)&user);
	}
	_txe_mutex_delete((TX_MUTEX *)&packetMutex);
}
